from ..base_settings import BaseModelSettings


class ChatModelSettings(BaseModelSettings):
    """
    Change these settings to tweak the model's behavior.

    Based on https://platform.openai.com/docs/api-reference/chat/create
    """
